package cc.niushuai.dididone.biz;

import android.content.Context;
import android.view.Gravity;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.loper7.date_time_picker.DateTimeConfig;
import com.loper7.date_time_picker.dialog.CardDatePickerDialog;
import com.mikepenz.iconics.IconicsDrawable;
import com.mikepenz.iconics.view.IconicsImageView;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import cc.niushuai.dididone.R;
import cc.niushuai.dididone.biz.entity.Project;
import cc.niushuai.dididone.biz.entity.Record;
import cc.niushuai.dididone.biz.roomx.DBManager;
import cc.niushuai.dididone.biz.vo.ProjectCount;
import cc.niushuai.dididone.util.IntSnowflake;
import cc.niushuai.dididone.util.XLog;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import io.reactivex.Flowable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;
import kotlin.Unit;
import kotlin.jvm.functions.Function1;

/**
 * 业务全局对象
 */
public class BizGlobal {
    public static final String REQUEST_CODE = "REQUEST_CODE";
    public static final String PROJECT_ID = "projectId";
    public static final String PROJECT_NAME = "projectName";
    public static final String PROJECT_ICON = "projectIcon";
    public static final String PROJECT_ICON_COLOR = "projectIconColor";
    public static final Map<Long, Project> CACHE_PROJECT = new HashMap<>();
    public static final Map<Long, Integer> CACHE_PROJECT_COUNT = new HashMap<>();
    public static final String EMPTY_PROJECT_TIPS = "先去添加打卡项吧~";
    public static final String EMPTY_PROJECT_TIPS_ICON = "cmd_alert_decagram_outline";
    public static final int REQUEST_CODE_GENERAL = 1;
    public static final int REQUEST_CODE_PROJECT_NEW = 10011;
    public static final int REQUEST_CODE_PROJECT_UPDATE = 10012;
    public static final Integer REQUEST_CODE_BACKUP = 10005;
    public static final Integer REQUEST_CODE_RESTORE = 10006;
    private static final String URI_SCHEMA_FILE = "file";
    private static final String URI_SCHEMA_CONTENT = "content";

    private BizGlobal() {
    }

    public static void buildCache() {

        if (!CACHE_PROJECT.isEmpty()) {
            return;
        }

        // 打卡项缓存
        buildProjectCache();

        // 按类别区分每一个打卡项 存在多少条打卡记录
        buildProjectCountCache();
    }

    private static void buildProjectCountCache() {
        Flowable<List<ProjectCount>> listFlowable = DBManager.INSTANCE.recordDao().groupCountProject();
        listFlowable.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(projectCountList -> {

                    for (ProjectCount projectCount : projectCountList) {
                        CACHE_PROJECT_COUNT.put(projectCount.getId(), projectCount.getCount());
                        XLog.d("build project {}:{} cache", projectCount.getName(), projectCount.getCount());
                    }
                    XLog.d("build project count cache size: {}", projectCountList.size());
                });
    }

    private static void buildProjectCache() {
        Flowable<List<Project>> listFlowable = DBManager.INSTANCE.projectDao().listAllWithDeleted();
        listFlowable.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(projectList -> {
                    for (Project project : projectList) {
                        CACHE_PROJECT.put(project.getId(), project);
                        XLog.d("build project {} cache", project.getName());
                    }

                    XLog.d("build project cache size: {}", projectList.size());
                });
    }

    public static Map<Long, Project> getAllProjectMap() {
        return CACHE_PROJECT;
    }

    public static Map<Long, Project> getAvailableProjectMap() {
        return CACHE_PROJECT.values().stream().filter(item -> item.getDeleted().equals(0)).collect(Collectors.toMap(Project::getId, Function.identity()));
    }

    public static List<Project> getAvailableProjectList() {
        return CACHE_PROJECT.values()
                .stream()
                .filter(item -> item.getDeleted().equals(0))
                .sorted((e1, e2) -> e2.getCreateDate().compareTo(e1.getCreateDate()))
                .collect(Collectors.toList());
    }

    public static RelativeLayout getItemLayout(Context context, Project project, Record record) {
        // 外层layout 白色背景 存在边距
        RelativeLayout itemLayout = new RelativeLayout(context);
        RelativeLayout.LayoutParams layoutParams = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.MATCH_PARENT, 175);
        layoutParams.setMargins(25, 20, 25, 15);

        int itemLayoutViewId = IntSnowflake.next_id();
        itemLayout.setId(itemLayoutViewId);
        itemLayout.setLayoutParams(layoutParams);
        itemLayout.setPadding(15, 15, 15, 5);
        itemLayout.setBackgroundResource(R.drawable.bg_miui10);
//        itemLayout.setOrientation(LinearLayout.VERTICAL);

        // projectIcon layout
        RelativeLayout.LayoutParams iconLayout = new RelativeLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        iconLayout.setMargins(10, 5, 0, 0);
        IconicsImageView iconView = new IconicsImageView(context);
        int iconViewId = IntSnowflake.next_id();
        iconView.setId(iconViewId);
        iconView.setLayoutParams(iconLayout);
        IconicsDrawable icon = new IconicsDrawable(context);
        icon.icon(project.getIcon()).color(project.getIconColor()).sizeDp(50);
        iconView.setIcon(icon);

        itemLayout.addView(iconView);

        // 打卡项名称
        RelativeLayout.LayoutParams projectLayout = new RelativeLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        projectLayout.setMargins(20, 5, 0, 0);
        projectLayout.addRule(RelativeLayout.RIGHT_OF, iconViewId);
        int projectNameViewId = IntSnowflake.next_id();
        // 打卡项名称文本view
        TextView projectTextView = getTextView(context, projectNameViewId, record.getName(), projectLayout, R.color._333333, 16, Gravity.CENTER_VERTICAL);
        itemLayout.addView(projectTextView);

        // 打卡时间
        RelativeLayout.LayoutParams checkDateLayout = new RelativeLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        checkDateLayout.setMargins(20, 25, 0, 0);
        checkDateLayout.addRule(RelativeLayout.RIGHT_OF, iconViewId);
        checkDateLayout.addRule(RelativeLayout.BELOW, projectNameViewId);
        int checkDateViewId = IntSnowflake.next_id();
        // 打卡时间文本view
        TextView checkDateTextView = getTextView(context, checkDateViewId, DateUtil.date(record.getCheckDate()).toString(DatePattern.NORM_DATE_PATTERN), checkDateLayout, R.color._666666, 12, Gravity.TOP);
        itemLayout.addView(checkDateTextView);

        // 创建时间
        RelativeLayout.LayoutParams createDateLayout = new RelativeLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        createDateLayout.setMargins(20, 25, 0, 0);
        createDateLayout.addRule(RelativeLayout.RIGHT_OF, checkDateViewId);
        createDateLayout.addRule(RelativeLayout.BELOW, projectNameViewId);
        createDateLayout.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);
        int createDateViewId = IntSnowflake.next_id();
        // 创建时间文本view
        TextView createDateTextView = getTextView(context, createDateViewId, DateUtil.date(record.getCreateDate()).toString(DatePattern.NORM_DATETIME_PATTERN), createDateLayout, R.color._666666, 12, Gravity.RIGHT);
        itemLayout.addView(createDateTextView);
        return itemLayout;
    }

    private static TextView getTextView(Context context, int id, String text, ViewGroup.LayoutParams textLayoutParams, int textColor, int textSize, int gravity) {
        TextView textView = new TextView(context);

        textView.setId(id);
        textView.setLayoutParams(textLayoutParams);
        textView.setText(text);
        textView.setTextColor(context.getResources().getColor(textColor));
        textView.setTextSize(textSize);
        textView.setGravity(gravity);
        return textView;
    }

    public static void buildCardDatePickerDialog(Context context, String title,
                                                 Long defaultDate, Long minDate, Long maxDate, boolean showTime,
                                                 String confirmTxt, Function1<Long, Unit> unitFunction) {

        CardDatePickerDialog.Builder builder = new CardDatePickerDialog.Builder(context);
        builder.setTitle(title)
                .setBackGroundModel(CardDatePickerDialog.STACK)
                .showBackNow(false)
                .setWrapSelectorWheel(false)
                .showDateLabel(true)
                .showFocusDateInfo(true)
                .setOnChoose(confirmTxt, unitFunction);

        if (null != defaultDate) {
            builder.setDefaultTime(defaultDate);
        }

        List<Integer> displayTypeList = new ArrayList<>(6);
        displayTypeList.add(DateTimeConfig.YEAR);
        displayTypeList.add(DateTimeConfig.MONTH);
        displayTypeList.add(DateTimeConfig.DAY);

        if (showTime) {
            displayTypeList.add(DateTimeConfig.HOUR);
            displayTypeList.add(DateTimeConfig.MIN);
            displayTypeList.add(DateTimeConfig.SECOND);
        }

        builder.setDisplayType(displayTypeList);

        if (null != minDate) {
            builder.setMinTime(minDate);
        }
        if (null != maxDate) {
            builder.setMaxTime(maxDate);
        }

        builder.build().show();
    }

    public static Long getTextViewDate(TextView textView) {
        return DateUtil.parseDate(textView.getText().toString()).getTime();
    }
}
