package cc.niushuai.dididone.biz.dao;

import androidx.room.Dao;
import androidx.room.Delete;
import androidx.room.Insert;
import androidx.room.Query;
import androidx.room.RoomWarnings;

import java.util.Iterator;
import java.util.List;

import cc.niushuai.dididone.biz.entity.Record;
import cc.niushuai.dididone.biz.vo.ProjectCount;
import io.reactivex.Completable;
import io.reactivex.Flowable;
import io.reactivex.Single;

@Dao
public interface RecordDao {

    @Query("SELECT * FROM t_record order by check_date desc, create_date desc")
    Flowable<List<Record>> listAll();

    @Query("SELECT * FROM t_record order by check_date asc, create_date desc")
    Single<List<Record>> xListAll();

    @Query("select * from t_record where deleted = 0 and check_date = :date order by check_date desc, create_date desc")
    Flowable<List<Record>> queryByDate(long date);

    @Query("select * from t_record where deleted = 0 and create_date >= :startDate and create_date <= :endDate order by check_date desc, create_date desc")
    Flowable<List<Record>> queryByDate(long startDate, long endDate);

    @Insert
    Completable insertAll(List<Record> records);

    @Insert
    Completable insertAll(Record... records);

    @Query("update t_record set deleted = 1 where id = :id")
    void deleteById(Long id);

    @Delete
    void delete(Record record);

    @Delete
    void delete(Record... records);

    @Query("select b.id, b.name, count(a.id) as count from t_record a left join t_project b on b.id = a.project_id and b.deleted = 0 where a.deleted = 0 group by b.id, b.name")
    Flowable<List<ProjectCount>> groupCountProject();

    @SuppressWarnings(RoomWarnings.CURSOR_MISMATCH)
    @Query("select check_date from t_record where deleted = 0 and check_date >= :startTime and check_date <= :endTime order by check_date asc")
    Flowable<List<Record>> queryPointByDate(long startTime, long endTime);

    @Query("select * from t_record where deleted = 0 and check_date >= :startDate and check_date <= :endDate order by check_date desc")
    Flowable<List<Record>> queryByCheckDate(long startDate, long endDate);
    @Query("select * from t_record where deleted = 0 and project_id = :projectId and check_date >= :startDate and check_date <= :endDate order by check_date desc")
    Flowable<List<Record>> queryByProjectAndDate(Long projectId, long startDate, long endDate);

    @Query("update t_record set deleted = 1 where project_id = :projectId")
    void deleteByProjectId(Long projectId);

    @Query("update t_record set name = :name where project_id = :projectId")
    int updateName(String name, Long projectId);
}
