package cc.niushuai.dididone.ui.base;

import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.view.LayoutInflaterCompat;

import com.mikepenz.community_material_typeface_library.CommunityMaterial;
import com.mikepenz.devicon_typeface_library.DevIcon;
import com.mikepenz.fontawesome_typeface_library.FontAwesome;
import com.mikepenz.foundation_icons_typeface_library.FoundationIcons;
import com.mikepenz.google_material_typeface_library.GoogleMaterial;
import com.mikepenz.iconics.Iconics;
import com.mikepenz.iconics.context.IconicsLayoutInflater2;
import com.mikepenz.iconics.typeface.IIcon;
import com.mikepenz.ionicons_typeface_library.Ionicons;

import java.util.ArrayList;
import java.util.List;

import cc.niushuai.dididone.biz.entity.SavedIcon;
import cc.niushuai.dididone.ui.setting.icon.recycle.IconRecycleAdapter;
import cc.niushuai.dididone.util.Snowflake;
import cc.niushuai.dididone.util.XLog;

public abstract class BaseActivity extends AppCompatActivity {

    protected void iconicsInit() {
        LayoutInflaterCompat.setFactory2(getLayoutInflater(), new IconicsLayoutInflater2(getDelegate()));
        Iconics.init(getApplicationContext());
        Iconics.registerFont(new DevIcon());
        Iconics.registerFont(new CommunityMaterial());
        Iconics.registerFont(new GoogleMaterial());
        Iconics.registerFont(new Ionicons());
        Iconics.registerFont(new FontAwesome());
        Iconics.registerFont(new FoundationIcons());

        // 初始化icon map
        addFontMap(DevIcon.Icon.class);
        addFontMap(CommunityMaterial.Icon.class);
        addFontMap(GoogleMaterial.Icon.class);
        addFontMap(Ionicons.Icon.class);
        addFontMap(FontAwesome.Icon.class);
        addFontMap(FoundationIcons.Icon.class);
    }

    private void addFontMap(Class<? extends IIcon> iconClass) {

        if (!iconClass.isEnum()) {
            XLog.d(this.getClass().getSimpleName(), iconClass.getName() + " class not enum class");
            return;
        }

        List<SavedIcon> items = new ArrayList<>();

        for (IIcon enumConstant : iconClass.getEnumConstants()) {
            SavedIcon icon = new SavedIcon();
            icon.setId(Snowflake.next_id());
            icon.setIcon(enumConstant.getName());
            icon.setColor(String.valueOf(Color.BLACK));
            icon.setSize(52);
            items.add(icon);
        }

        IconRecycleAdapter.ICON_MAP.put(IconRecycleAdapter.getIconClassName(iconClass), items);
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        // 初始化字体
        iconicsInit();

        super.onCreate(savedInstanceState);
    }

    @Override
    protected void onPostCreate(@Nullable Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);

        // 初始化基础数据
        init();
        // 给控件增加监听器
        setListeners();
    }

    /**
     * 初始化方法
     *
     * @author niushuai
     * @date: 2022/10/19 15:54
     */
    protected abstract void init();

    /**
     * 监听器
     */
    protected abstract void setListeners();
}