package cc.niushuai.dididone.ui.calendar;

import android.app.AlertDialog;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import com.github.clans.fab.FloatingActionButton;
import com.github.clans.fab.FloatingActionMenu;
import com.lxj.xpopup.XPopup;
import com.lxj.xpopup.interfaces.OnConfirmListener;
import com.mikepenz.iconics.IconicsDrawable;
import com.mikepenz.iconics.view.IconicsImageView;
import com.mikepenz.ionicons_typeface_library.Ionicons;
import com.necer.calendar.BaseCalendar;
import com.necer.entity.Lunar;
import com.necer.enumeration.DateChangeBehavior;
import com.necer.listener.OnCalendarChangedListener;
import com.necer.painter.InnerPainter;
import com.necer.utils.CalendarUtil;

import org.joda.time.LocalDate;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.ISODateTimeFormat;
import org.reactivestreams.Subscription;

import java.util.List;
import java.util.stream.Collectors;

import cc.niushuai.dididone.R;
import cc.niushuai.dididone.biz.BizGlobal;
import cc.niushuai.dididone.biz.entity.Project;
import cc.niushuai.dididone.biz.entity.Record;
import cc.niushuai.dididone.biz.roomx.DBManager;
import cc.niushuai.dididone.databinding.FragmentCalendarBinding;
import cc.niushuai.dididone.ui.base.BaseFragment;
import cc.niushuai.dididone.ui.component.ProjectPopupView;
import cc.niushuai.dididone.util.IntSnowflake;
import cc.niushuai.dididone.util.Toasts;
import cc.niushuai.dididone.util.XLog;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import io.reactivex.Completable;
import io.reactivex.CompletableObserver;
import io.reactivex.Flowable;
import io.reactivex.FlowableSubscriber;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;

public class CalendarFragment extends BaseFragment {

    private FragmentCalendarBinding binding;

    // 默认当天
    private String lastDate;

//    private List<Project> floatingActionProjectList;

    public View onCreateView(@NonNull LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        XLog.d("{} onCreateView", getClass().getSimpleName());
        CalendarViewModel calendarViewModel = new ViewModelProvider(this).get(CalendarViewModel.class);

        binding = FragmentCalendarBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        return root;
    }

    @Override
    public void init() {
        initPointList();

        XLog.d("lastDate: {}", lastDate);
        if (StrUtil.isNotEmpty(lastDate)) {
            binding.miui10Calendar.setInitializeDate(lastDate);
            rebuildRecordListViewByDate(lastDate);

            LocalDate localDate = LocalDate.parse(lastDate, ISODateTimeFormat.date());
            setLunarDateLabel(localDate);
        }

        // 构建弹出框内容
//        fetchNewestActionButtonData();

        // check entry
    }

    private void initPointList() {

        // 获取当前月视图中存在record的日期
        DateTime currMonth;
        if (null == lastDate) {
            currMonth = DateUtil.date();
        } else {
            currMonth = DateUtil.parseDate(lastDate);
        }

        long startTime = DateUtil.beginOfMonth(currMonth).getTime();
        long endTime = DateUtil.endOfMonth(currMonth).getTime();

        Flowable<List<Record>> listFlowable = DBManager.INSTANCE.recordDao().queryPointByDate(startTime, endTime);
        listFlowable.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(recordList -> {
                    List<String> checkDateList = recordList.stream()
                            .map(Record::getCheckDate)
                            .map(item -> DateUtil.date(item).toDateStr())
                            .collect(Collectors.toList());
                    if (binding == null) {
                        return;
                    }

                    InnerPainter painter = (InnerPainter) binding.miui10Calendar.getCalendarPainter();
                    painter.setPointList(checkDateList);
                });

    }

    @Override
    public void setListeners() {
        // 日历选择变更事件
        addCalendarChangeListener();
        // floatAction
//        addFloatActionMenuToggleListener();

        //
        addCheckEntryListener();

    }

    private void addCheckEntryListener() {

        binding.checkEntry.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // 按钮置灰
//                binding.checkEntry.setImageDrawable(new IconicsDrawable(getContext())
//                        .icon(CommunityMaterial.Icon.cmd_checkbox_marked_circle)
//                        .color(getContext().getColor(R.color.black_less))
//                        .sizeDp(85)
//                );

                // 弹窗
                new XPopup.Builder(getContext())
                        .hasShadowBg(true)
                        .isViewMode(true)
                        .isDestroyOnDismiss(true)
                        .asCustom(new ProjectPopupView(getContext(), project -> {
                            // item click callback
                            add2Top(project);
                        }))
                        .show();

                // 还原按钮
//                binding.checkEntry.setImageDrawable(new IconicsDrawable(getContext())
//                        .icon(CommunityMaterial.Icon.cmd_checkbox_marked_circle)
//                        .color(getContext().getColor(R.color.black))
//                        .sizeDp(85)
//                );
            }
        });

    }

//    private void addFloatActionMenuToggleListener() {
//        binding.didiCheckMenu.setOnMenuToggleListener(new FloatingActionMenu.OnMenuToggleListener() {
//            @Override
//            public void onMenuToggle(boolean opened) {
//                if (opened) {
//                    buildFloatActionButtons(floatingActionProjectList, getContext(), binding.didiCheckMenu);
//                } else {
//                    // 关闭时清除所有button
//                    binding.didiCheckMenu.removeAllMenuButtons();
//                }
//            }
//        });
//    }

    private void buildFloatActionButtons(List<Project> floatingActionProjectList, Context context, FloatingActionMenu didiCheckMenu) {
        for (Project project : floatingActionProjectList) {

            FloatingActionButton actionButton = new FloatingActionButton(context);
            actionButton.showButtonInMenu(true);
            actionButton.show(true);
            actionButton.setTag(project.getName() + project.getId());
            actionButton.setLabelText(project.getName());
            actionButton.setImageDrawable(new IconicsDrawable(context).icon(project.getIcon()).color(project.getIconColor()).sizeDp(32));
            actionButton.setButtonSize(FloatingActionButton.SIZE_NORMAL);
            actionButton.setColorNormal(getResources().getColor(R.color.white));
            actionButton.setColorPressed(getResources().getColor(R.color.white));

            actionButton.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    // 打卡完成 关闭动画
                    didiCheckMenu.close(true);
                    // 添加数据
//                    tabsPagerAdapter.getHomeFragment().add2Top(CheckTypeEnum.QUICK_ADD, actionButton.getLabelText());

                }
            });

            didiCheckMenu.addMenuButton(actionButton);
        }
    }


    /**
     * 日历选择变更事件
     */
    private void addCalendarChangeListener() {

        binding.miui10Calendar.setOnCalendarChangedListener(new OnCalendarChangedListener() {
            @Override
            public void onCalendarChange(BaseCalendar baseCalendar, int year, int month, LocalDate localDate, DateChangeBehavior dateChangeBehavior) {
                if (null != lastDate && lastDate.equals(localDate.toString())) {
                    return;
                }

                XLog.d("当前选中 " + localDate);
                setLunarDateLabel(localDate);

                // 重组日期
                rebuildRecordListViewByDate(lastDate);
            }
        });

    }

    private void setLunarDateLabel(LocalDate localDate) {
        Lunar lunar = CalendarUtil.getCalendarDate(localDate).lunar;
        String lunarDate = lunar.chineseEra + "年" + lunar.lunarMonthStr + lunar.lunarDayStr;

        // 重置上次选中日期
        lastDate = localDate.toString();

        binding.tvResult.setText(StrUtil.format("{} | {}", lastDate, lunarDate));
    }

    private void rebuildRecordListViewByDate(String date) {
        DateTime dateTime = DateUtil.parseDate(date);
        // 移除所有旧内容项
        binding.cCalendarCheckInList.removeAllViews();
        // 添加新内容项
        Flowable<List<Record>> flowable = DBManager.INSTANCE.recordDao().queryByDate(dateTime.getTime());
        flowable.subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).subscribe(new FlowableSubscriber<List<Record>>() {
            @Override
            public void onSubscribe(Subscription s) {
                // 3s超时
                s.request(1);
            }

            @Override
            public void onNext(List<Record> recordList) {
                XLog.d("DataListOnNext 查询到数据: {}", recordList.size());

                for (Record record : recordList) {
                    Project project = BizGlobal.getAllProjectMap().get(record.getProjectId());
                    if (null == project) {
                        Toasts.longShow(getContext(), "ProjectId: {}不存在, 可能存在BUG", String.valueOf(record.getProjectId()));
                        return;
                    }
                    add2Top(project, record);
                }
            }

            @Override
            public void onError(Throwable t) {
                Toasts.shortShow(getActivity(), "DataList根据时间拉取历史数据失败: ", t.getLocalizedMessage());
            }

            @Override
            public void onComplete() {
            }
        });

    }

    private void add2Top(Project project) {
        // 过滤掉empty选项
        if (BizGlobal.EMPTY_PROJECT_TIPS.equals(project.getName())) {
            return;
        }

        Record record = new Record();
        record.setProjectId(project.getId());
        record.setName(project.getName());
        record.setCheckDate(DateUtil.parseDate(lastDate).getTime());

        // 插入数据库
        Completable completable = DBManager.INSTANCE.recordDao().insertAll(record);
        completable.subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).subscribe(new CompletableObserver() {
            @Override
            public void onSubscribe(Disposable d) {
            }

            @Override
            public void onComplete() {
                XLog.i("[{}]打卡成功", project.getName());
                Toasts.shortShow(getActivity(), "[{}]打卡成功", project.getName());

                // 更新视图
                add2Top(project, record);
            }

            @Override
            public void onError(Throwable e) {
                XLog.e("[{}]打卡失败, {}", project.getName(), e);
                Toasts.shortShow(getActivity(), "[{}]打卡失败, {}", project.getName(), e.getMessage());
            }
        });
    }

    private void add2Top(Project project, Record record) {
        RelativeLayout itemLayout = BizGlobal.getItemLayout(getContext(), project, record);

        itemLayout.setOnLongClickListener(new View.OnLongClickListener() {
            @Override
            public boolean onLongClick(View v) {

                XLog.d("project: {} {} {} record: {} {} {} long clicked", project.getName(), project.getIcon(), record.getId(),
                        record.getProjectId(), record.getName(), record.getCheckDate());

                AlertDialog alertDialog = new AlertDialog.Builder(getContext())
                        .setTitle(StrUtil.format("确定要删除 [{}]", record.getName()))
                        .setIcon(new IconicsDrawable(getContext()).icon(Ionicons.Icon.ion_android_warning).color(Color.RED))
                        .setPositiveButton("确定", (dialog, which) -> {
                            // 删除
                            deleteRecord(project, record);
                            // 更新视图
                            binding.cCalendarCheckInList.removeView(itemLayout);
                            binding.cCalendarCheckInList.requestLayout();
                        })
                        .setNegativeButton("取消", ((dialog, which) -> dialog.dismiss()))
                        .create();
                alertDialog.show();

                return true;
            }
        });

//        binding.cCalendarCheckInList.removeAllViews();
        binding.cCalendarCheckInList.addView(itemLayout, 0);
        binding.cCalendarCheckInList.requestLayout();
    }

    private void deleteRecord(Project project, Record record) {
        //
        DBManager.INSTANCE.recordDao().deleteById(record.getId());
    }


    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }

//
//    private void fetchNewestActionButtonData() {
//        Flowable<List<Project>> listFlowable = DBManager.INSTANCE.projectDao().listAll();
//        listFlowable.subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread()).subscribe(projectList -> floatingActionProjectList = projectList);
//    }
}