package cc.niushuai.dididone.ui.component;

import android.content.Context;
import android.graphics.Color;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.lxj.easyadapter.EasyAdapter;
import com.lxj.easyadapter.MultiItemTypeAdapter;
import com.lxj.easyadapter.ViewHolder;
import com.lxj.xpopup.core.BottomPopupView;
import com.lxj.xpopup.util.XPopupUtils;
import com.lxj.xpopup.widget.VerticalRecyclerView;
import com.mikepenz.iconics.IconicsDrawable;

import java.util.List;
import java.util.function.Consumer;

import cc.niushuai.dididone.R;
import cc.niushuai.dididone.biz.BizGlobal;
import cc.niushuai.dididone.biz.entity.Project;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;

public class ProjectPopupView extends BottomPopupView {

    private VerticalRecyclerView recyclerView;
    private List<Project> data;
    private EasyAdapter<Project> commonAdapter;

    private Consumer<Project> callback;


    public ProjectPopupView(@NonNull Context context) {
        super(context);
    }

    public ProjectPopupView(@NonNull Context context, Consumer<Project> callback) {
        super(context);
        this.callback = callback;
    }

    @Override
    protected int getImplLayoutId() {
        return R.layout.project_popup_view;
    }

    @Override
    protected void onCreate() {
        super.onCreate();

        recyclerView = findViewById(R.id.ppv_recyclerView);
        // 构建data
        data = CollUtil.list(false, BizGlobal.getAvailableProjectMap().values());
        data.sort((e1, e2) -> e2.getCreateDate().compareTo(e1.getCreateDate()));

        int projectCount = data.size();

        if (data.isEmpty()) {
            Project e = new Project();
            e.setName(BizGlobal.EMPTY_PROJECT_TIPS);
            e.setIcon(BizGlobal.EMPTY_PROJECT_TIPS_ICON);
            e.setIconColor(Color.RED);
            data.add(e);
            projectCount = 0;
        }

        ((TextView) findViewById(R.id.ppv_title)).setText(StrUtil.format("全部{}条打卡项", projectCount));
        commonAdapter = new EasyAdapter<Project>(data, R.layout.project_popup_view_item) {
            @Override
            protected void bind(@NonNull ViewHolder holder, @NonNull Project item, final int position) {

                Project project = data.get(position);

                ImageView iconView = holder.getView(R.id.ppv_item_icon);
                iconView.setImageDrawable(
                        new IconicsDrawable(getContext())
                                .icon(project.getIcon())
                                .color(project.getIconColor())
                                .sizeDp(50));

                TextView txtView = holder.getView(R.id.ppv_item_txt);
                txtView.setText(project.getName());

                TextView descView = holder.getView(R.id.ppv_item_desc);
                String count = BizGlobal.CACHE_PROJECT_COUNT.getOrDefault(project.getId(), 0).toString();
                descView.setText(StrUtil.format("累计打卡{}次", count));
            }
        };
        commonAdapter.setOnItemClickListener(new MultiItemTypeAdapter.OnItemClickListener() {
            @Override
            public void onItemClick(@NonNull View view, @NonNull RecyclerView.ViewHolder viewHolder, int position) {

                // 打卡
                callback.accept(data.get(position));

                dismiss();
            }

            @Override
            public boolean onItemLongClick(@NonNull View view, @NonNull RecyclerView.ViewHolder viewHolder, int position) {
                return true;
            }
        });

        recyclerView.setAdapter(commonAdapter);
    }

    @Override
    protected void onShow() {
        super.onShow();
    }

    @Override
    protected void onDismiss() {
        super.onDismiss();
    }

    @Override
    protected int getMaxHeight() {
        return (int) (XPopupUtils.getScreenHeight(getContext()) * .5f);
    }
}
