package cc.niushuai.dididone.ui.setting;

import android.Manifest;
import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.google.gson.JsonParser;

import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import cc.niushuai.dididone.R;
import cc.niushuai.dididone.biz.BizGlobal;
import cc.niushuai.dididone.biz.entity.Project;
import cc.niushuai.dididone.biz.entity.ProjectRecord;
import cc.niushuai.dididone.biz.entity.Record;
import cc.niushuai.dididone.biz.roomx.DBManager;
import cc.niushuai.dididone.databinding.FragmentSettingBinding;
import cc.niushuai.dididone.ui.base.BaseFragment;
import cc.niushuai.dididone.util.GsonUtil;
import cc.niushuai.dididone.util.Toasts;
import cc.niushuai.dididone.util.XLog;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import io.reactivex.Completable;
import io.reactivex.CompletableObserver;
import io.reactivex.Single;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;

public class SettingFragment extends BaseFragment {

    private static final String FILE_NAME = "{}_backup_{}.json";
    private FragmentSettingBinding binding;

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {

        binding = FragmentSettingBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        return root;
    }

    @Override
    public void setListeners() {

        // 备份点击监听事件
        addBackupClickListener();
        // 恢复点击监听事件
        addRestoreClickListener();

    }

    private void addRestoreClickListener() {
        binding.sSetRestore.setOnClickListener(view -> {
            requestForPermission(BizGlobal.REQUEST_CODE_RESTORE, Intent.ACTION_GET_CONTENT);
        });
    }

    private void addBackupClickListener() {

        binding.sSetBackup.setOnClickListener(view -> {
            requestForPermission(BizGlobal.REQUEST_CODE_BACKUP, Intent.ACTION_CREATE_DOCUMENT);
        });
    }

    private void requestForPermission(Integer requestCode, String action) {
        if (
            // true表示未授权读权限
                ContextCompat.checkSelfPermission(getContext(), Manifest.permission.READ_EXTERNAL_STORAGE) == PackageManager.PERMISSION_DENIED ||
                        // true表示未授权写权限
                        ContextCompat.checkSelfPermission(getContext(), Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_DENIED) {
            // 申请授权
            if (ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), Manifest.permission.READ_EXTERNAL_STORAGE)
                    || ActivityCompat.shouldShowRequestPermissionRationale(getActivity(), Manifest.permission.WRITE_EXTERNAL_STORAGE)
            ) {
                Toasts.longShow(getContext(), "您曾经选择过禁止弹窗授权, 请手动进入应用管理授权");
            } else {
                ActivityCompat.requestPermissions(getActivity(), new String[]{
                        Manifest.permission.READ_EXTERNAL_STORAGE,
                        Manifest.permission.WRITE_EXTERNAL_STORAGE,
                }, BizGlobal.REQUEST_CODE_GENERAL);
            }
        } else {
            chooseBackupFileLocation(requestCode, action);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {

        boolean isAllGranted = false;


        for (int i = 0; i < permissions.length; i++) {
            XLog.d("permission: {} result: {}", permissions[i], grantResults[i]);

            if (grantResults[i] == PackageManager.PERMISSION_DENIED) {
                isAllGranted = false;
//                        break;
            }
        }

        if (isAllGranted) {
            // 授权通过时 打开文件保存逻辑

            String action = null;

            if (BizGlobal.REQUEST_CODE_BACKUP.equals(requestCode)) {
                action = Intent.ACTION_CREATE_DOCUMENT;
            } else if (BizGlobal.REQUEST_CODE_RESTORE.equals(requestCode)) {
                action = Intent.ACTION_GET_CONTENT;
            } else {
                XLog.d("unknown request code: {}", requestCode);
                Toasts.shortShow(getContext(), "未知的请求码: {}", requestCode + "");
                return;
            }

            chooseBackupFileLocation(requestCode, action);
        }

    }

    private void chooseBackupFileLocation(Integer requestCode, String action) {
        // 写文件
        Intent chooseFileLocationIntent = new Intent(action);
        chooseFileLocationIntent.addCategory(Intent.CATEGORY_OPENABLE);
        chooseFileLocationIntent.setType("*/*");
        if (BizGlobal.REQUEST_CODE_BACKUP.equals(requestCode)) {
            chooseFileLocationIntent.putExtra(Intent.EXTRA_TITLE, StrUtil.format(FILE_NAME, getString(R.string.app_name), DateUtil.date().toString(DatePattern.PURE_DATETIME_PATTERN)));
        }

        // 选择文件 在onActivityResult中监听选择的文件
        startActivityForResult(chooseFileLocationIntent, requestCode);
    }

    private String dealBackupContent(List<Record> recordList) {

        // 组装实体类
        Map<Long, Project> allProjectMap = BizGlobal.getAllProjectMap();
        List<ProjectRecord> projectRecordList = new ArrayList<>(allProjectMap.size());
        for (Long projectId : allProjectMap.keySet()) {
            List<Record> collect = recordList.stream().filter(item -> projectId.equals(item.getProjectId())).collect(Collectors.toList());
            projectRecordList.add(new ProjectRecord(allProjectMap.get(projectId), collect));
        }
        // JSON化
        return GsonUtil.toJson(projectRecordList);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        if (resultCode != Activity.RESULT_OK) {
            Toasts.shortShow(getContext(), "操作失败 {} --> {}", requestCode + "", resultCode + "");
            return;
        }
        if (BizGlobal.REQUEST_CODE_BACKUP.equals(requestCode)) {
            backupResult(data);
        } else if (BizGlobal.REQUEST_CODE_RESTORE.equals(requestCode)) {
            restoreResult(data);
        }
    }

    private void restoreResult(Intent data) {

        try {
            Uri uri = data.getData();
            InputStream inputStream = getContext().getContentResolver().openInputStream(uri);

            String readStr = IoUtil.read(inputStream, StandardCharsets.UTF_8);
            if (!GsonUtil.isJson(readStr)) {
                XLog.d("非json: {}", readStr);
                Toasts.shortShow(getContext(), "文件内容非JSON格式");
                return;
            }

            List<Map<String, Object>> list = GsonUtil.toBean(readStr, List.class);
            if (CollUtil.isEmpty(list)) {
                Toasts.shortShow(getContext(), "未发现可导入内容, 请检查文件是否正确");
                return;
            }

            List<ProjectRecord> projectRecordList = new ArrayList<>(list.size());
            for (Map<String, Object> map : list) {
                String json = GsonUtil.toJson(map);
                ProjectRecord projectRecord = GsonUtil.toBean(json, ProjectRecord.class);
                projectRecordList.add(projectRecord);
            }

            restoreProjectRecord(projectRecordList);

        } catch (FileNotFoundException e) {
            XLog.d("文件读取失败: {}", e.getMessage(), e);
            Toasts.shortShow(getContext(), "文件读取失败: {}", e.getMessage());
        }
    }

    private void restoreProjectRecord(List<ProjectRecord> projectRecordList) {

        if (CollUtil.isNotEmpty(projectRecordList)) {

            insertProject(projectRecordList);

            insertRecord(projectRecordList);
        }


    }

    private void insertRecord(List<ProjectRecord> projectRecordList) {
        for (ProjectRecord projectRecord : projectRecordList) {

            List<Record> recordList = projectRecord.getRecordList();
            if (CollUtil.isNotEmpty(recordList)) {
                // 插入record
                Completable completable = DBManager.INSTANCE.recordDao().insertAll(recordList);
                completable.subscribeOn(Schedulers.io())
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe(new CompletableObserver() {
                            @Override
                            public void onSubscribe(Disposable d) {
                                XLog.d("restore insertRecord onSubscribe");
                            }

                            @Override
                            public void onComplete() {
                                XLog.d("restore insertRecord onComplete");
                                BizGlobal.buildCache();
                                Toasts.shortShow(getContext(), "恢复打卡记录成功");
                            }

                            @Override
                            public void onError(Throwable e) {
                                XLog.d("restore insertRecord onError: {}", e.getMessage(), e);
                                Toasts.shortShow(getContext(),"恢复打卡记录出错: {}", e.getMessage());
                            }
                        });
            }

        }
    }

    private void insertProject(List<ProjectRecord> projectRecordList) {
        List<Project> projectList = projectRecordList.stream().map(item -> {
            Project project = new Project();
            BeanUtil.copyProperties(item, project);
            return project;
        }).collect(Collectors.toList());

        // 插入project
        Completable completable = DBManager.INSTANCE.projectDao().insertAll(projectList);
        completable.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new CompletableObserver() {
                    @Override
                    public void onSubscribe(Disposable d) {
                        XLog.d("restore insertProject onComplete");
                    }

                    @Override
                    public void onComplete() {
                        XLog.d("restore insertProject onComplete");
                        BizGlobal.buildCache();
                        Toasts.shortShow(getContext(), "恢复打卡项成功");
                    }

                    @Override
                    public void onError(Throwable e) {
                        XLog.d("restore insertProject onError: {}", e.getMessage(), e);
                        Toasts.shortShow(getContext(),"恢复打卡项出错: {}", e.getMessage());
                    }
                });
    }

    private void backupResult(@NonNull Intent data) {
        Uri fileUri = data.getData();

        Single<List<Record>> listSingle = DBManager.INSTANCE.recordDao().xListAll();
        listSingle.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe((recordList, throwable) -> {
                    XLog.d("addBackClickListener == " + recordList.size());

                    try {
                        String content = dealBackupContent(recordList);
                        OutputStream outputStream = getContext().getContentResolver().openOutputStream(fileUri);
                        IoUtil.write(outputStream, true, content.getBytes(StandardCharsets.UTF_8));

                        Toasts.shortShow(getContext(), "文件已备份: {}", fileUri.getPath().substring(fileUri.getPath().indexOf(":")).replace(":", "/sdcard/"));
                    } catch (FileNotFoundException e) {
                        XLog.d("save file【{}】 error: {}", fileUri.getPath(), e.getMessage(), e);
                    }
                });
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }
}