package cc.niushuai.dididone.ui.setting.icon.recycle;

import android.app.Activity;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.mikepenz.iconics.IconicsDrawable;
import com.mikepenz.iconics.typeface.IIcon;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import cc.niushuai.dididone.R;
import cc.niushuai.dididone.biz.entity.SavedIcon;
import cc.niushuai.dididone.databinding.IconGridItemBinding;
import cc.niushuai.dididone.util.XLog;

public class IconRecycleAdapter extends RecyclerView.Adapter<IconRecycleAdapter.IconRecycleViewHolder> {

    public static final Map<String, List<SavedIcon>> ICON_MAP = new HashMap<>();
    int selectedPosition = -1;
    int lastSelectedPosition = -1;
    private List<SavedIcon> allIconList;
    private List<SavedIcon> savedIconList;
    private Activity activity;
    private String selectedIcon;

    public IconRecycleAdapter(Activity activity) {
        this.activity = activity;
        this.allIconList = getIconValues();
        change(item -> true);
    }

    public static String getIconClassName(Class<? extends IIcon> iconClass) {
        String iconClassName = iconClass.getName();
        return iconClassName.substring(iconClassName.lastIndexOf(".") + 1, iconClassName.indexOf("$"));

    }

    public String getSelectedIcon() {
        return selectedIcon;
    }

    public void change(Predicate<? super SavedIcon> filter) {
        this.savedIconList = allIconList.stream().filter(filter).collect(Collectors.toList());
        this.notifyDataSetChanged();
    }

    @NonNull
    private static List<SavedIcon> getIconValues() {
        List<SavedIcon> list = new ArrayList<>();
        ICON_MAP.values().forEach(item -> list.addAll(item));
        return list;
    }

    /**
     * @param parent   The ViewGroup into which the new View will be added after it is bound to
     *                 an adapter position.
     * @param viewType The view type of the new View.
     * @return
     */
    @NonNull
    @Override
    public IconRecycleViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
//        View view = View.inflate(activity, R.layout.icon_grid_item, null);
//        return new IconRecycleViewHolder(view);
        return new IconRecycleViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.icon_grid_item, parent, false));
    }

    /**
     * @param holder   The ViewHolder which should be updated to represent the contents of the
     *                 item at the given position in the data set.
     * @param position The position of the item within the adapter's data set.
     */
    @Override
    public void onBindViewHolder(@NonNull IconRecycleViewHolder holder, int position) {
        SavedIcon icon = savedIconList.get(position);
        holder.iconView.setImageDrawable(new IconicsDrawable(activity)
                .icon(icon.getIcon())
                .color(Integer.parseInt(icon.getColor()))
                .sizeDp(icon.getSize()));

        holder.txtView.setText(icon.getIcon());

        holder.binding.getRoot().setOnClickListener(v -> {
            lastSelectedPosition = selectedPosition;
            selectedPosition = holder.getBindingAdapterPosition();
            notifyItemChanged(lastSelectedPosition);
            notifyItemChanged(selectedPosition);
        });
        if (selectedPosition == holder.getBindingAdapterPosition()) {
            holder.binding.iconsGridLayout.setBackgroundColor(activity.getColor(R.color.theme_color_primary));
        } else {
            holder.binding.iconsGridLayout.setBackgroundColor(Color.WHITE);
        }

        selectedIcon = icon.getIcon();
        XLog.d("onItemClick position: {}, icon: {}, selectedIcon: {}", position, icon.getIcon(), selectedIcon);
    }

    /**
     * @return
     */
    @Override
    public int getItemCount() {
        return savedIconList.size();
    }

    public class IconRecycleViewHolder extends RecyclerView.ViewHolder {

        protected ImageView iconView;
        protected TextView txtView;
        private IconGridItemBinding binding;


        public IconRecycleViewHolder(@NonNull View itemView) {
            super(itemView);
            binding = IconGridItemBinding.bind(itemView);

            iconView = itemView.findViewById(R.id.icons_grid_item);
            txtView = itemView.findViewById(R.id.icons_grid_txt);
        }
    }

}
