package cc.niushuai.dididone.ui.setting.project;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Gravity;
import android.view.View;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.DefaultItemAnimator;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.mikepenz.iconics.IconicsDrawable;
import com.mikepenz.ionicons_typeface_library.Ionicons;
import com.skydoves.colorpickerview.ColorEnvelope;
import com.skydoves.colorpickerview.ColorPickerDialog;
import com.skydoves.colorpickerview.ColorPickerView;
import com.skydoves.colorpickerview.listeners.ColorEnvelopeListener;
import com.skydoves.colorpickerview.listeners.ColorListener;

import cc.niushuai.dididone.R;
import cc.niushuai.dididone.biz.BizGlobal;
import cc.niushuai.dididone.biz.vo.ProjectIcon;
import cc.niushuai.dididone.databinding.ActivityNewProjectBinding;
import cc.niushuai.dididone.ui.base.BaseActivity;
import cc.niushuai.dididone.ui.setting.icon.recycle.IconRecycleAdapter;
import cc.niushuai.dididone.util.Toasts;
import cc.niushuai.dididone.util.XLog;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;

public class NewProjectActivity extends BaseActivity {

    private ActivityNewProjectBinding binding;
    private int selectedColor = Color.RED;
    private String selectedIcon = Ionicons.Icon.ion_ios_heart.name();

    private Long projectId;
    private Integer requestCode;

    /**
     * icon 展示 recycleViewAdapter
     */
    private IconRecycleAdapter iconRecycleAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);

        binding = ActivityNewProjectBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
    }



    @Override
    protected void init() {

        // 是否需要restore数据
        boolean update = restore();
        // 设置标题
        setTitle(update ? "更新打卡项" : "添加打卡项");

    }

    private boolean restore() {

        Bundle extras = getIntent().getExtras();
        requestCode = extras.getInt(BizGlobal.REQUEST_CODE);
        long id = extras.getLong(BizGlobal.PROJECT_ID);
        String name = extras.getString(BizGlobal.PROJECT_NAME);
        String icon = extras.getString(BizGlobal.PROJECT_ICON);
        int iconColor = extras.getInt(BizGlobal.PROJECT_ICON_COLOR);

        // 重新赋值
        projectId = id;
        selectedIcon = icon;
        selectedColor = iconColor;

        // 修改text内容
        binding.npProjectNameTxt.setText(name);
        changeIcon(icon, iconColor);

        return BizGlobal.REQUEST_CODE_PROJECT_NEW == requestCode ? false : true;
    }

    private void initIconDialogView(View iconDialogView) {

        initIconDialogRecyclerView(iconDialogView);

//        initIconDialogSpinner(iconDialogView);

        addIconDialogSearchTxtListener(iconDialogView);
    }

    private void addIconDialogSearchTxtListener(View iconDialogView) {

        TextView iconDialogSearchTxtView = iconDialogView.findViewById(R.id.icons_dialog_search);
        iconDialogSearchTxtView.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                iconRecycleAdapter.change(item -> item.getIcon().contains(s.toString()));
            }
        });
    }

    private void initIconDialogRecyclerView(View iconDialogView) {
        RecyclerView iconDialogRecyclerView = iconDialogView.findViewById(R.id.icons_dialog_recyclerView);

        iconDialogRecyclerView.addItemDecoration(new DividerItemDecoration(this, DividerItemDecoration.VERTICAL));
        DefaultItemAnimator itemAnimator = new DefaultItemAnimator();
        itemAnimator.setAddDuration(1000);
        itemAnimator.setRemoveDuration(1000);
        iconDialogRecyclerView.setItemAnimator(itemAnimator);

        LinearLayoutManager layoutManager = new LinearLayoutManager(this);
        iconDialogRecyclerView.setLayoutManager(layoutManager);

        iconRecycleAdapter = new IconRecycleAdapter(this);
        iconDialogRecyclerView.setAdapter(iconRecycleAdapter);
    }

    private void initIconDialogSpinner(View iconDialogView) {
//        Set<String> keySet = IconRecycleAdapter.ICON_MAP.keySet();
//        ArrayList<String> keys = new ArrayList<>(keySet);
//        Collections.sort(keys);
//
//        Spinner iconDialogSpinner = iconDialogView.findViewById(R.id.icons_dialog_select);
//        iconDialogSpinner.setAdapter(new ArrayAdapter<String>(this, com.necer.R.layout.support_simple_spinner_dropdown_item, keys));
//        iconDialogSpinner.setSelection(0);
//
//        iconDialogSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
//            @Override
//            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
//                String str = ((TextView) iconDialogView.findViewById(R.id.icons_dialog_search)).getText().toString();
//                Predicate<SavedIcon> filter = item -> true;
//                if (null != str && str.length() > 0) {
//                    filter = item -> item.getIcon().contains(str);
//                }
//                iconRecycleAdapter.change(filter);
//            }
//
//            @Override
//            public void onNothingSelected(AdapterView<?> parent) {
//            }
//        });
    }

    @Override
    protected void setListeners() {

        // icon选择器
        iconClickListener();

        // 颜色选择器
        colorPickListener();

        // 返回按钮
        binding.npButtonBack.setOnClickListener(view -> finish());

        // 确定按钮
        binding.npButtonConfirm.setOnClickListener(view -> {
            // 三种属性
            Intent data = new Intent();
            // 名称
            String projectName = binding.npProjectNameTxt.getText().toString();
            if (StrUtil.isEmpty(projectName)) {
                Toasts.shortShow(this, "{}未填", getResources().getString(R.string.np_project_name));
                return;
            }
            // id
            if (null != projectId) {
                data.putExtra(BizGlobal.PROJECT_ID, projectId);
            }
            // 名称
            data.putExtra(BizGlobal.PROJECT_NAME, projectName);
            // 图标
            data.putExtra(BizGlobal.PROJECT_ICON, selectedIcon);
            // 颜色
            data.putExtra(BizGlobal.PROJECT_ICON_COLOR, selectedColor);
            // requestCode
            data.putExtra(BizGlobal.REQUEST_CODE, requestCode);
            // 放置结果数据
            setResult(Activity.RESULT_OK, data);
            finish();
        });
    }

    private void colorPickListener() {
        binding.npColorPicker.setOnClickListener(view -> {

            ColorPickerDialog.Builder builder = new ColorPickerDialog.Builder(this);

            TextView title = new TextView(this);
            title.setText("选择一个图标颜色");
            title.setPadding(10, 30, 0, 30);
            title.setGravity(Gravity.CENTER_VERTICAL);
            title.setTextSize(24);
            title.setTextColor(Color.BLACK);

            ColorPickerView colorPickerView = builder.getColorPickerView();
            colorPickerView.setInitialColor(selectedColor);
            colorPickerView.setColorListener(new ColorListener() {
                @Override
                public void onColorSelected(int color, boolean fromUser) {
                    selectedColor = color;
                    title.setBackgroundColor(color);
                }
            });

            builder.setCustomTitle(title)
                    .setPositiveButton(getString(R.string.cp_color_confirm),
                            new ColorEnvelopeListener() {
                                @Override
                                public void onColorSelected(ColorEnvelope envelope, boolean fromUser) {
//                                    setLayoutColor(envelope);
                                    XLog.d("选择的颜色：{} {} {}, fromUser: {}", envelope.getColor(), envelope.getArgb(), envelope.getHexCode(), fromUser);
                                    selectedColor = envelope.getColor();

                                    // 改变图标以及颜色
                                    changeIcon(null, selectedColor);
                                }
                            })
                    .setNegativeButton(getString(R.string.cancel), (dialogInterface, i) -> {
                        colorPickerView.setInitialColor(selectedColor);
                        dialogInterface.dismiss();
                    })
                    .attachAlphaSlideBar(true)
                    .attachBrightnessSlideBar(true)
                    .setBottomSpace(12);

            builder.create().show();
        });
    }

    private void iconClickListener() {

        binding.npIconFill.setOnClickListener(view -> {

            View iconDialogView = View.inflate(this, R.layout.fragment_icon_dialog, null);
            // 初始化 并且添加监听事件
            initIconDialogView(iconDialogView);

            AlertDialog.Builder builder = new AlertDialog.Builder(this).setTitle("选择一个图标");
            AlertDialog alertDialog = builder.setView(iconDialogView)
                    .setNegativeButton("返回", (dialog, which) -> dialog.dismiss())
                    .setPositiveButton("选择该图标", (dialog, which) -> {
                        selectedIcon = iconRecycleAdapter.getSelectedIcon();
                        changeIcon(selectedIcon, selectedColor);
                    }).create();

            alertDialog.show();
        });
    }

    /**
     * 改变图标以及颜色
     *
     * @param icon  图标
     * @param color 颜色
     */
    private void changeIcon(String icon, int color) {
        if (StrUtil.isEmpty(icon)) {
            icon = ReflectUtil.getFieldValue(binding.npIconFill.getDrawable(), "mIcon").toString();
        }
        binding.npIconFill.setImageDrawable(new IconicsDrawable(this)
                .icon(icon)
                .color(color)
                .sizeDp(50)
        );
    }

}