package cc.niushuai.dididone.ui.setting.project;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;

import com.mikepenz.iconics.IconicsDrawable;
import com.mikepenz.ionicons_typeface_library.Ionicons;

import java.util.ArrayList;
import java.util.List;

import cc.niushuai.dididone.biz.BizGlobal;
import cc.niushuai.dididone.biz.entity.Project;
import cc.niushuai.dididone.biz.roomx.DBManager;
import cc.niushuai.dididone.biz.vo.ProjectIcon;
import cc.niushuai.dididone.databinding.FragmentProjectBinding;
import cc.niushuai.dididone.ui.base.InitAndSetListener;
import cc.niushuai.dididone.util.Toasts;
import cc.niushuai.dididone.util.XLog;
import cn.hutool.core.util.StrUtil;
import io.reactivex.Completable;
import io.reactivex.CompletableObserver;
import io.reactivex.Flowable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;

public class ProjectFragment extends Fragment implements InitAndSetListener {

    private FragmentProjectBinding binding;
    private GridViewAdapter4Project gridViewAdapter4Project;

    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        binding = FragmentProjectBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        init();
        setListeners();

        return root;
    }

    /**
     *
     */
    @Override
    public void init() {

        initProjectGrid(getDataList());
    }

    private void initProjectGrid(List<ProjectIcon> dataList) {
        gridViewAdapter4Project = new GridViewAdapter4Project(getContext(), dataList);
        binding.gridProject.setAdapter(gridViewAdapter4Project);
    }

    private List<ProjectIcon> getDataList() {
        ArrayList<ProjectIcon> list = new ArrayList<>();

        Flowable<List<Project>> listFlowable = DBManager.INSTANCE.projectDao().listAll();
        listFlowable.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(projectList -> {
                    XLog.d("reset... project 查询到" + projectList.size());

                    list.clear();
                    list.addAll(ProjectIcon.trans(projectList));
                    // 最后一个要添加 + 符号
                    list.add(defaultAddProject());

                    initProjectGrid(list);
                });

        return list;
    }

    private ProjectIcon defaultAddProject() {
        return new ProjectIcon(0L, "添加", "faw_plus", Color.BLACK);
    }

    /**
     *
     */
    @Override
    public void setListeners() {

        // 添加点击事件
        this.addGridItemClickListener();
    }

    private void addGridItemClickListener() {

        binding.gridProject.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {

                if (id == 0) {
                    addProjectClick(BizGlobal.REQUEST_CODE_PROJECT_NEW, null);
                    return;
                }

                // 其他的被点击
                ProjectIcon item = gridViewAdapter4Project.getItem(position);
//                Toasts.shortShow(getContext(), item.getProjectName());

                addProjectClick(BizGlobal.REQUEST_CODE_PROJECT_UPDATE, item);
            }
        });


        // 长按事件  用于删除
        binding.gridProject.setOnItemLongClickListener(new AdapterView.OnItemLongClickListener() {
            @Override
            public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {

                if (id == 0) {
                    // 添加被点击 跳过
                    return false;
                }

                ProjectIcon item = gridViewAdapter4Project.getItem(position);

                AlertDialog alertDialog = new AlertDialog.Builder(getContext())
                        .setTitle(StrUtil.format("确定要删除 [{}]", item.getProjectName()))
                        .setIcon(new IconicsDrawable(getContext()).icon(Ionicons.Icon.ion_android_warning).color(Color.RED))
                        .setPositiveButton("确定", (dialog, which) -> {


                            // 是否级联删除打卡项
                            confirmDeleteProjectRecord(item);
                            XLog.d("删除 {} 打卡项", item.getProjectName());
                            // 删除
                            deleteProject(item);
                            // 更新视图
                            gridViewAdapter4Project.projectIconList.remove(position);
                            gridViewAdapter4Project.notifyDataSetChanged();
                        })
                        .setNegativeButton("取消", ((dialog, which) -> dialog.dismiss()))
                        .create();
                alertDialog.show();

                return true;
            }

            private void confirmDeleteProjectRecord(ProjectIcon item) {
                // 再次弹出是否级联删除选项
                AlertDialog projectItemAlertDialog = new AlertDialog.Builder(getContext())
                        .setTitle(StrUtil.format("是否要删除 [{}] 已打卡记录?", item.getProjectName()))
                        .setIcon(new IconicsDrawable(getContext()).icon(Ionicons.Icon.ion_android_warning).color(Color.RED))
                        .setPositiveButton("确定", (dialog, which) -> {

                            XLog.d("删除 {} 已打卡记录", item.getProjectName());
                            DBManager.INSTANCE.recordDao().deleteByProjectId(item.getId());
                        })
                        .setNegativeButton("取消", ((dialog, which) -> dialog.dismiss()))
                        .create();
                projectItemAlertDialog.show();
            }
        });
    }

    /**
     * 删除该项内容
     *
     * @param item
     */
    private void deleteProject(ProjectIcon item) {
        // 删除
        DBManager.INSTANCE.projectDao().deleteById(item.getId());
        // remove by id
        BizGlobal.CACHE_PROJECT.get(item.getId()).setDeleted(1);
        XLog.d("删除项{}, id: {}", item.getProjectName(), item.getId());
    }

    /**
     * 添加 被点击
     */
    private void addProjectClick(int requestCode, ProjectIcon clickItem) {
        // 打开添加新项页面
        Intent intent = new Intent(getContext(), NewProjectActivity.class);
        Bundle data = new Bundle();
        if (null != clickItem) {
            // requestCode
            intent.putExtra(BizGlobal.REQUEST_CODE, requestCode);
            // id
            intent.putExtra(BizGlobal.PROJECT_ID, clickItem.getId());
            // 名称
            intent.putExtra(BizGlobal.PROJECT_NAME, clickItem.getProjectName());
            // 图标
            intent.putExtra(BizGlobal.PROJECT_ICON, clickItem.getIcon());
            // 颜色
            intent.putExtra(BizGlobal.PROJECT_ICON_COLOR, clickItem.getColor());
        }
        startActivityForResult(intent, requestCode, data);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {

        if (resultCode != Activity.RESULT_OK || null == data) {
            return;
        }

        boolean update = requestCode == BizGlobal.REQUEST_CODE_PROJECT_UPDATE ? true : false;

        // id
        long projectId = data.getLongExtra(BizGlobal.PROJECT_ID, -1L);
        XLog.d(BizGlobal.PROJECT_ID + ": {}", projectId);
        // 名称
        String projectName = data.getStringExtra(BizGlobal.PROJECT_NAME);
        XLog.d(BizGlobal.PROJECT_NAME + ": {}", projectName);
        // 图标
        String projectIcon = data.getStringExtra(BizGlobal.PROJECT_ICON);
        XLog.d(BizGlobal.PROJECT_ICON + ": {}", projectIcon);
        // 颜色
        int projectIconColor = data.getIntExtra(BizGlobal.PROJECT_ICON_COLOR, Color.RED);
        XLog.d(BizGlobal.PROJECT_ICON_COLOR + ": {}", projectIconColor);

        Project project = new Project();
        project.setName(projectName);
        project.setIcon(projectIcon);
        project.setIconColor(projectIconColor);

        // 是否要添加id选项
        if (update) {
            project.setId(projectId);
        }

        XLog.d("update ? {}", update);
        if (update) {
            // 更新
            int rows = DBManager.INSTANCE.projectDao().update(project);
            Toasts.shortShow(getContext(), "rows = " + rows);
            XLog.d("update project rows == " + rows);
            if (rows == 1) {
                rows = DBManager.INSTANCE.recordDao().updateName(project.getName(), project.getId());
                XLog.d("update record rows == " + rows);
            }
        } else {
            // 新增
            Completable completable = DBManager.INSTANCE.projectDao().insertAll(project);
            completable.subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new CompletableObserver() {
                        @Override
                        public void onSubscribe(Disposable d) {
                        }

                        @Override
                        public void onComplete() {
                            Log.i("SettingsFragment", "新增打卡项[" + project.getName() + "]成功");
                            Toasts.shortShow(getActivity(), "新增打卡项[{}]成功", project.getName());
                        }

                        @Override
                        public void onError(Throwable e) {
                            Log.e("SettingsFragment", "新增打卡项[" + project.getName() + "]失败", e);
                            Toasts.shortShow(getActivity(), "新增打卡项[{}]失败, {}", project.getName(), e.getMessage());
                        }
                    });

        }
    }
}
