package cc.niushuai.dididone.ui.statistic;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.fragment.app.FragmentTransaction;
import androidx.lifecycle.ViewModelProvider;

import cc.niushuai.dididone.R;
import cc.niushuai.dididone.databinding.FragmentStatisticBinding;
import cc.niushuai.dididone.ui.base.BaseFragment;
import cc.niushuai.dididone.ui.statistic.charts.ListFragment;
import cc.niushuai.dididone.ui.statistic.charts.MonthFragment;
import cc.niushuai.dididone.ui.statistic.charts.OtherFragment;
import cc.niushuai.dididone.ui.statistic.charts.WeekFragment;
import cc.niushuai.dididone.util.XLog;

public class StatisticFragment extends BaseFragment implements View.OnClickListener {

    private FragmentStatisticBinding binding;

    private ListFragment listFragment;
    private WeekFragment weekFragment;
    private MonthFragment monthFragment;
    private OtherFragment otherFragment;

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        XLog.d("{} onCreateView", getClass().getSimpleName());
        StatisticViewModel statisticViewModel =
                new ViewModelProvider(this).get(StatisticViewModel.class);

        binding = FragmentStatisticBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        return root;
    }

    @Override
    public void init() {
        initListFragment();
    }

    @Override
    public void setListeners() {
        binding.statButtonList.setOnClickListener(this);
        binding.statButtonWeek.setOnClickListener(this);
        binding.statButtonMonth.setOnClickListener(this);
        binding.statButtonOther.setOnClickListener(this);
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }

    /**
     * list维度
     */
    private void initListFragment() {
        FragmentTransaction transaction = getActivity().getSupportFragmentManager().beginTransaction();

        if (listFragment == null) {
            listFragment = new ListFragment();
            transaction.add(R.id.stat_main_frame_layout, listFragment);
        }
        hideFragment(transaction);
        transaction.show(listFragment);

        transaction.commit();
    }

    /**
     * 周维度
     */
    private void initWeekFragment() {
        FragmentTransaction transaction = getActivity().getSupportFragmentManager().beginTransaction();

        if (weekFragment == null) {
            weekFragment = new WeekFragment();
            transaction.add(R.id.stat_main_frame_layout, weekFragment);
        }
        hideFragment(transaction);
        transaction.show(weekFragment);

        transaction.commit();
    }

    /**
     * 月维度视图
     */
    private void initMonthFragment() {
        FragmentTransaction transaction = getActivity().getSupportFragmentManager().beginTransaction();

        if (monthFragment == null) {
            monthFragment = new MonthFragment();
            transaction.add(R.id.stat_main_frame_layout, monthFragment);
        }
        hideFragment(transaction);
        transaction.show(monthFragment);

        transaction.commit();
    }

    /**
     * 其他
     */
    private void initOtherFragment() {
        FragmentTransaction transaction = getActivity().getSupportFragmentManager().beginTransaction();

        if (otherFragment == null) {
            otherFragment = new OtherFragment();
            transaction.add(R.id.stat_main_frame_layout, otherFragment);
        }
        hideFragment(transaction);
        transaction.show(otherFragment);

        transaction.commit();
    }

    //隐藏所有的fragment
    private void hideFragment(FragmentTransaction transaction) {
        if (listFragment != null) {
            transaction.hide(listFragment);
        }
        if (weekFragment != null) {
            transaction.hide(weekFragment);
        }
        if (monthFragment != null) {
            transaction.hide(monthFragment);
        }
        if (otherFragment != null) {
            transaction.hide(otherFragment);
        }
    }

    @Override
    public void onClick(View v) {
        if (v == binding.statButtonList) {
            initListFragment();
        } else if (v == binding.statButtonWeek) {
            initWeekFragment();
        } else if (v == binding.statButtonMonth) {
            initMonthFragment();
        } else if (v == binding.statButtonOther) {
            initOtherFragment();
        }
    }
}