package cc.niushuai.dididone.ui.statistic.charts;

import static cc.niushuai.dididone.biz.BizGlobal.getTextViewDate;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.RelativeLayout;
import android.widget.Spinner;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.util.List;
import java.util.stream.Collectors;

import cc.niushuai.dididone.R;
import cc.niushuai.dididone.biz.BizGlobal;
import cc.niushuai.dididone.biz.entity.Project;
import cc.niushuai.dididone.biz.entity.Record;
import cc.niushuai.dididone.biz.roomx.DBManager;
import cc.niushuai.dididone.databinding.FragmentStatListBinding;
import cc.niushuai.dididone.util.XLog;
import cn.hutool.core.date.DateUtil;
import io.reactivex.Flowable;
import io.reactivex.schedulers.Schedulers;
import kotlin.Unit;
import kotlin.jvm.functions.Function1;

public class ListFragment extends BaseChartFragment {

    private FragmentStatListBinding binding;

    public ListFragment() {
        super("列表视图");
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        XLog.d("{} onCreateView", getTitle());
        View view = inflater.inflate(R.layout.fragment_stat_list, container, false);
        binding = FragmentStatListBinding.bind(view);
        return binding.getRoot();
    }

    @Override
    public void init() {

        // 初始化初始日期
        initDate(binding.statListStartDate, binding.statListEndDate);
        // 初始化spinner
        initSpinner(binding.statListSpinnerProject);
    }

    private void initDate(TextView startDate, TextView endDate) {
        String today = DateUtil.date().toDateStr();
        startDate.setText(today);
        endDate.setText(today);
    }

    private void initSpinner(Spinner spinner) {

        List<String> items = BizGlobal.getAvailableProjectList().stream().map(item -> item.getName()).collect(Collectors.toList());
        items.add(0, "选择打卡项");
        spinner.setAdapter(new ArrayAdapter<>(getContext(), androidx.appcompat.R.layout.support_simple_spinner_dropdown_item, items));

        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {

                onConditionChange(position, binding.statListStartDate.getText(), binding.statListEndDate.getText());
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
            }
        });
        spinner.setSelection(0);
    }

    private void onConditionChange(int position, CharSequence startDate, CharSequence endDate) {
        position--;
        Flowable<List<Record>> listFlowable;
        if (position < 0) {
            // position小于0 表示选中第一个 即未选择打卡项 查询所有打卡项
            listFlowable = DBManager.INSTANCE.recordDao().queryByCheckDate(dateStr2mills(startDate.toString()), dateStr2mills(endDate.toString()));
        } else {
            listFlowable = DBManager.INSTANCE.recordDao().queryByProjectAndDate(
                    BizGlobal.getAvailableProjectList().get(position).getId(),
                    dateStr2mills(startDate.toString()),
                    dateStr2mills(endDate.toString()));
        }


        listFlowable.subscribeOn(Schedulers.io()).subscribe(recordList -> {
            XLog.d("recordList: {}", recordList.size());
            recordList.forEach(item -> XLog.d("{} at {}", item.getName(), DateUtil.date(item.getCheckDate()).toDateStr()));

            onRecordDataList(recordList);
        });
    }

    private void onRecordDataList(List<Record> recordList) {

        getActivity().runOnUiThread(() -> {
            binding.statTotalMsg.setText("总计: " + recordList.size() + "条记录");
            binding.statListCheckInList.removeAllViews();
        });

        // 更新视图
        for (int i = recordList.size() - 1; i >= 0; i--) {
            // 查出来的数据是倒叙的
            Record record = recordList.get(i);
            getActivity().runOnUiThread(() -> add2Top(BizGlobal.CACHE_PROJECT.get(record.getProjectId()), record));
        }
    }

    private void add2Top(Project project, Record record) {

        RelativeLayout itemLayout = BizGlobal.getItemLayout(getContext(), project, record);

        binding.statListCheckInList.addView(itemLayout, 0);
        binding.statListSpinnerProject.requestLayout();
    }

    @Override
    public void setListeners() {

        binding.statListStartDate.setOnClickListener(v -> {
            Function1<Long, Unit> unitFunction = millisecond -> {
                String dateStr = DateUtil.date(millisecond).toDateStr();
                XLog.d("select date = {}", dateStr);

                binding.statListStartDate.setText(dateStr);

                onConditionChange(binding.statListSpinnerProject.getSelectedItemPosition(), binding.statListStartDate.getText(), binding.statListEndDate.getText());

                return null;
            };

            BizGlobal.buildCardDatePickerDialog(getContext(), "选择起始日期", getTextViewDate(binding.statListStartDate), null, getTextViewDate(binding.statListEndDate),
                    false, "确定", unitFunction);
        });

        binding.statListEndDate.setOnClickListener(v -> {
            Function1<Long, Unit> unitFunction = millisecond -> {
                String dateStr = DateUtil.date(millisecond).toDateStr();
                XLog.d("select date = {}", dateStr);

                binding.statListEndDate.setText(dateStr);

                onConditionChange(binding.statListSpinnerProject.getSelectedItemPosition(), binding.statListStartDate.getText(), binding.statListEndDate.getText());

                return null;
            };

            BizGlobal.buildCardDatePickerDialog(getContext(), "选择结束日期", getTextViewDate(binding.statListEndDate), getTextViewDate(binding.statListStartDate), null,
                    false, "确定", unitFunction);
        });

    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }
}